﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using Buzz.MachineInterface;

namespace NTracker
{
	[MachineDecl(Name = "Jeskola NTracker", ShortName = "NTracker", Author = "Oskari Tammelin", MaxTracks = 32)]
	public class NTrackerMachine : IBuzzMachine, INotifyPropertyChanged
	{
		IBuzzMachineHost host;
		Track[] tracks = new Track[32];

		public NTrackerMachine(IBuzzMachineHost host)
		{
			this.host = host;
			
			for (int i = 0; i < tracks.Length; i++)
				tracks[i] = new Track(host);

		}

		[ParameterDecl(IsStateless = true)]
		public void Note(Note v, int track) { tracks[track].Note(v); }

		[ParameterDecl(IsStateless = true, IsWaveNumber = true)]
		public void Wave(int v, int track) { tracks[track].Wave(v); }

		[ParameterDecl(IsStateless = true, MinValue = 0, MaxValue = 127, DefValue = 100)]
		public void Velocity(int v, int track) { tracks[track].Velocity(v); }

		public bool Work(Sample[] output, int n, WorkModes mode)
		{
			bool gotOutput = false;

			for (int i = 0; i < n; i++)
				output[i] = 0.0f;

			for (int t = 0; t < host.Machine.ParameterGroups[2].TrackCount; t++)
				gotOutput |= tracks[t].Work(output, n);

			return gotOutput;
		}

		public event PropertyChangedEventHandler PropertyChanged;
	}
}
