﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buzz.MachineInterface;

namespace NTracker
{
	class Track
	{
		IBuzzMachineHost host;
		
		Note note;
		int waveNumber = 1;
		int velocity = 100;
		bool trigger;

		List<Voice> voices = new List<Voice>();

		public Track(IBuzzMachineHost host)
		{
			this.host = host;
		}

		public void Note(Note v)
		{
			note = v;
			trigger = true;
		}

		public void Wave(int v)
		{
			waveNumber = v;
		}

		public void Velocity(int v)
		{
			velocity = v;
		}

		void Trigger()
		{
			if (voices.Count > 0) voices.Last().Cut();

			if (note.Value != 255)
			{
				voices.Add(new Voice(host, note, velocity, waveNumber));
			}
		}

		public bool Work(Sample[] output, int n)
		{
			if (trigger)
			{
				Trigger();
				trigger = false;
			}

			bool hadVoices = voices.Count > 0;

			foreach (var v in voices)
				v.Work(output, n);

			for (int i = voices.Count - 1; i >= 0; i--)
			{
				if (voices[i].IsFinished)
					voices.RemoveAt(i);
			}

			return hadVoices;
		}
	}
}
