﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buzz.MachineInterface;
using BuzzGUI.Interfaces;

namespace NTracker
{
	class Voice
	{
		IBuzzMachineHost host;

		Note note;
		int velocity;
		int waveNumber;
		double position;
		int rampPosition = -1;
		int rampLength;
		float gain;

		public bool IsFinished { get; private set; }

		public Voice(IBuzzMachineHost host, Note note, int velocity, int wavenum)
		{
			this.host = host;
			this.note = note;
			this.velocity = velocity;
			this.waveNumber = wavenum;
			gain = velocity * velocity / (127.0f * 127.0f);
		}

		public void Cut()
		{
			if (rampPosition < 0)
			{
				rampLength = host.MsToSamples(5.0f);
				rampPosition = 0;
			}
		}

		public void Work(Sample[] output, int n)
		{
			var wd = new WaveData(host, waveNumber);
			
			var f = wd.GetFrequency(note, host.MasterInfo.SamplesPerSec);

			for (int i = 0; i < n; i++)
				output[i] += gain * Resample(wd, f) * Ramp();
		}

		Sample Resample(WaveData wd, double frequency)
		{
			var s = new Sample();
			if (wd.IsEmpty) return s;

			if (position >= wd.SampleCount)
			{
				IsFinished = true;
				return s;
			}

			int ipos = (int)position;
			float alpha = (float)(position - ipos);

			var s0 = wd.GetSample(ipos);
			var s1 = wd.GetSample(ipos + 1);
			s = s0 + (s1 - s0) * alpha;
			
			position += frequency;

			return s;
		}
	
		float Ramp()
		{
			if (rampPosition >= 0)
			{
				if (rampPosition++ < rampLength)
				{
					return (float)Math.Cos(Math.PI * rampPosition / rampLength) * 0.5f + 0.5f;
				}
				else
				{
					IsFinished = true;
					return 0.0f;
				}
			}

			return 1.0f;
		}

	}
}
