﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buzz.MachineInterface;
using BuzzGUI.Interfaces;

namespace NTracker
{
	unsafe class WaveData
	{
		void* pvsamples;
		int channelCount;
		int sampleCount;
		WaveFormat format;
		int sampleRate;
		Note rootNote;

		public bool IsEmpty { get { return pvsamples == null; } }
		public int SampleCount { get { return sampleCount; } }

		public WaveData(IBuzzMachineHost host, int wavenum)
		{
			var wt = host.Machine.Graph.Buzz.Song.Wavetable;
			var wave = wt.Waves[wavenum - 1];
			var wl = wave != null ? wave.Layers[0] : null;
			if (wl != null)
			{
				// cache all required data locally (very important for good performance)
				pvsamples = wl.RawSamples.ToPointer();
				channelCount = wl.ChannelCount;
				sampleCount = wl.SampleCount;
				format = wl.Format;
				sampleRate = wl.SampleRate;
				rootNote = new Note(wl.RootNote);
			}
		}

		public double GetFrequency(Note note, int outputRate)
		{
			return rootNote.Value != 0 ? Math.Pow(2.0, (note.ToMIDINote() - rootNote.ToMIDINote()) / 12.0) * sampleRate / outputRate : 0.0;
		}

		public Sample GetSample(int ipos)
		{
			var s = new Sample();
			if (ipos >= sampleCount) return s;

			switch (format)
			{
				case WaveFormat.Int16:
					{
						short* psamples = (short*)pvsamples;

						if (channelCount == 1)
						{
							s.L = s.R = psamples[ipos];
						}
						else
						{
							s.L = psamples[2 * ipos + 0];
							s.R = psamples[2 * ipos + 1];
						}
					}
					break;

				case WaveFormat.Float32:
					{
						float* psamples = (float*)pvsamples;

						if (channelCount == 1)
						{
							s.L = s.R = psamples[ipos] * 32768.0f;
						}
						else
						{
							s.L = psamples[2 * ipos + 0] * 32768.0f;
							s.R = psamples[2 * ipos + 1] * 32768.0f;
						}
					}
					break;

				case WaveFormat.Int32:
					{
						int* psamples = (int*)pvsamples;

						if (channelCount == 1)
						{
							s.L = s.R = (float)psamples[ipos] * (1.0f / 65536.0f);
						}
						else
						{
							s.L = (float)psamples[2 * ipos + 0] * (1.0f / 65536.0f);
							s.R = (float)psamples[2 * ipos + 1] * (1.0f / 65536.0f);
						}
					}
					break;

				case WaveFormat.Int24:
					{

						if (channelCount == 1)
						{
							int* p = (int*)((byte*)pvsamples + 3 * ipos);
							s.L = s.R = (float)(*p << 8 >> 8) * (1.0f / 256.0f);
						}
						else
						{
							int* pl = (int*)((byte*)pvsamples + 6 * ipos);
							s.L = (float)(*pl << 8 >> 8) * (1.0f / 256.0f);
							int* pr = (int*)((byte*)pvsamples + 6 * ipos + 3);
							s.R = (float)(*pr << 8 >> 8) * (1.0f / 256.0f);
						}
					}
					break;

			}

			return s;
		}

	}
}
